#!/usr/bin/perl -w

use strict;
use Test::More tests => 15;

use_ok( "IO::Async::MergePoint" );

use_ok( "IO::Async::Notifier" );
use_ok( "IO::Async::Handle" );
use_ok( "IO::Async::Stream" );
use_ok( "IO::Async::Timer" );
use_ok( "IO::Async::Timer::Countdown" );
use_ok( "IO::Async::Timer::Periodic" );
use_ok( "IO::Async::Signal" );
use_ok( "IO::Async::Listener" );

use_ok( "IO::Async::Loop::Select" );
use_ok( "IO::Async::Loop::Poll" );

use_ok( "IO::Async::Test" );

use_ok( "IO::Async::DetachedCode" );
use_ok( "IO::Async::Resolver" );
use_ok( "IO::Async::Connector" );
